##############################################################################
#
# Copyright (c) 2007 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
$Id:$
"""
__docformat__ = "reStructuredText"


class Error(Exception):
    """Base class for client errors."""
    def __str__(self):
        return repr(self)


class ProtocolError(Error):
    """Indicates an HTTP protocol error.

    Indicates an HTTP-level protocol error.  This is raised by the HTTP
    transport layer, if the server returns an error code other than 200
    (OK).

    @param url The target URL.
    @param errcode The HTTP error code.
    @param errmsg The HTTP error message.
    @param headers The HTTP header dictionary.

    """
    def __init__(self, url, errcode, errmsg, headers):
        Error.__init__(self)
        self.url = url
        self.errcode = errcode
        self.errmsg = errmsg
        self.headers = headers
    def __repr__(self):
        return (
            "<ProtocolError for %s: %s %s>" %
            (self.url, self.errcode, self.errmsg)
            )


class ResponseError(ValueError):
    """Indicates a broken response package.

    Indicates a broken JSON-RPC response package.  This exception is
    raised by the unmarshalling layer, if the JSON-RPC response is
    malformed.
    
    """
    pass


# TODO: implement this option if we implement __jsonclass__, ri
#class Fault(Error):
#    """Indicates an JSON-RPC fault package.
#
#    Indicates an JSON-RPC fault response package.  This exception is
#    raised by the unmarshalling layer, if the JSON-RPC response contains
#    a fault string.  This exception can also used as a class, to
#    generate a fault JSON-RPC message.
#
#    @param faultCode The JSON-RPC fault code.
#    @param faultString The JSON-RPC fault string.
#
#    """
#    def __init__(self, faultCode, faultString, **extra):
#        Error.__init__(self)
#        self.faultCode = faultCode
#        self.faultString = faultString
#    def __repr__(self):
#        return (
#            "<Fault %s: %s>" %
#            (self.faultCode, repr(self.faultString))
#            )



# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2007 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
$Id:$
"""
__docformat__ = "reStructuredText"

import unittest
import BaseHTTPServer
import random
import threading
import urllib
import cgi
import pprint

import zope.component
from zope.testing import doctest
from zope.testing.doctestunit import DocFileSuite

from z3c.json import minjson as json
from z3c.json.minjson import ReadException
from z3c.json.minjson import WriteException

from z3c.json.interfaces import IJSONWriter, IJSONReader
from z3c.json.converter import JSONWriter, JSONReader
from z3c.json.exceptions import ResponseError, ProtocolError

from z3c.json.proxy import JSONRPCProxy
from z3c.json.transport import BasicAuthTransport


def spaceless(aString):
    return aString.replace(' ','')


class JSONTests(unittest.TestCase):

    def testReadString(self):
        s = u"'hello'"
        self.assertEqual(json.read(s) ,'hello')

    def testWriteString(self):
        s = 'hello'
        self.assertEqual(json.write(s), '"hello"')

    def testReadInt(self):
        s = u"1"
        self.assertEqual(json.read(s), 1)

    def testWriteInt(self):
        s = 1
        self.assertEqual(json.write(s), "1")

    def testReadLong(self):
        s = u"999999999999999999999"
        self.assertEqual(json.read(s), 999999999999999999999)

    def testWriteShortLong(self):
        s = 1L
        self.assertEqual(json.write(s), "1")

    def testWriteLongLong(self):
        s = 999999999999999999999L
        self.assertEqual(json.write(s), "999999999999999999999")

    def testReadNegInt(self):
        s = u"-1"
        assert json.read(s) == -1

    def testWriteNegInt(self):
        s = -1
        assert json.write(s) == '-1'

    def testReadFloat(self):
        s = u"1.334"
        assert json.read(s) == 1.334

    def testReadEFloat1(self):
        s = u"1.334E2"
        assert json.read(s) == 133.4

    def testReadEFloat2(self):
        s = u"1.334E-02"
        assert json.read(s) == 0.01334

    def testReadeFloat1(self):
        s = u"1.334e2"
        assert json.read(s) == 133.4

    def testReadeFloat2(self):
        s = u"1.334e-02"
        assert json.read(s) == 0.01334

    def testWriteFloat(self):
        s = 1.334
        assert json.write(s) == "1.334"

    def testWriteDecimal(self):
        try:
            from decimal import Decimal
            s = Decimal('1.33')
            assert json.write(s) == "1.33"
        except ImportError:
            pass

    def testReadNegFloat(self):
        s = u"-1.334"
        assert json.read(s) == -1.334

    def testWriteNegFloat(self):
        s = -1.334
        assert json.write(s) == "-1.334"

    def testReadEmptyDict(self):
        s = u"{}"
        assert json.read(s) == {}

    def testWriteEmptyList(self):
        s = []
        assert json.write(s) == "[]"

    def testWriteEmptyTuple(self):
        s = ()
        assert json.write(s) == "[]"

    def testReadEmptyList(self):
        s = u"[]"
        assert json.read(s) == []

    def testWriteEmptyDict(self):
        s = {}
        assert json.write(s) == '{}'

    def testReadTrue(self):
        s = u"true"
        assert json.read(s) == True

    def testWriteTrue(self):
        s = True
        assert json.write(s) == "true"

    def testReadStringTrue(self):
        s = u'"true"'
        assert json.read(s) == 'true'

    def testWriteStringTrue(self):
        s = "True"
        assert json.write(s) == '"True"'

    def testReadStringNull(self):
        s = u'"null"'
        assert json.read(s) == 'null'

    def testWriteStringNone(self):
        s = "None"
        assert json.write(s) == '"None"'

    def testReadFalse(self):
        s = u"false"
        assert json.read(s) == False

    def testWriteFalse(self):
        s = False
        assert json.write(s) == 'false'

    def testReadNull(self):
        s = u"null"
        assert json.read(s) == None

    def testWriteNone(self):
        s = None
        assert json.write(s) == "null"

    def testReadDictOfLists(self):
        s = u"{'a':[],'b':[]}"
        assert json.read(s) == {'a':[],'b':[]}

    def testReadDictOfListsWithSpaces(self):
        s = u"{  'a' :    [],  'b'  : []  }    "
        assert json.read(s) == {'a':[],'b':[]}

    def testWriteDictOfLists(self):
        s = {'a':[],'b':[]}
        assert spaceless(json.write(s)) == '{"a":[],"b":[]}'

    def testWriteDictOfTuples(self):
        s = {'a':(),'b':()}
        assert spaceless(json.write(s)) == '{"a":[],"b":[]}'

    def testWriteDictWithNonemptyTuples(self):
        s = {'a':('fred',7),'b':('mary',1.234)}
        w = json.write(s)
        assert spaceless(w) == '{"a":["fred",7],"b":["mary",1.234]}'

    def testWriteVirtualTuple(self):
        s = 4,4,5,6
        w = json.write(s)
        assert spaceless(w) == '[4,4,5,6]'

    def testReadListOfDicts(self):
        s = u"[{},{}]"
        assert json.read(s) == [{},{}]

    def testReadListOfDictsWithSpaces(self):
        s = u" [ {    } ,{   \n} ]   "
        assert json.read(s) == [{},{}]

    def testWriteListOfDicts(self):
        s = [{},{}]
        assert spaceless(json.write(s)) == "[{},{}]"

    def testWriteTupleOfDicts(self):
        s = ({},{})
        assert spaceless(json.write(s)) == "[{},{}]"

    def testReadListOfStrings(self):
        s = u"['a','b','c']"
        assert json.read(s) == ['a','b','c']

    def testReadListOfStringsWithSpaces(self):
        s = u" ['a'    ,'b'  ,\n  'c']  "
        assert json.read(s) == ['a','b','c']

    def testReadStringWithWhiteSpace(self):
        s = ur"'hello \tworld'"
        assert json.read(s) == 'hello \tworld'

    def testWriteMixedList(self):
        o = ['OIL',34,199L,38.5]
        assert spaceless(json.write(o)) == '["OIL",34,199,38.5]'

    def testWriteMixedTuple(self):
        o = ('OIL',34,199L,38.5)
        assert spaceless(json.write(o)) == '["OIL",34,199,38.5]'

    def testWriteStringWithWhiteSpace(self):
        s = 'hello \tworld'
        assert json.write(s) == r'"hello \tworld"'

    def testWriteListofStringsWithApostrophes(self):
        s = ["hasn't","don't","isn't",True,"won't"]
        w = json.write(s)
        assert spaceless(w) == '["hasn\'t","don\'t","isn\'t",true,"won\'t"]'

    def testWriteTupleofStringsWithApostrophes(self):
        s = ("hasn't","don't","isn't",True,"won't")
        w = json.write(s)
        assert spaceless(w) == '["hasn\'t","don\'t","isn\'t",true,"won\'t"]'

    def testWriteListofStringsWithRandomQuoting(self):
        s = ["hasn't","do\"n't","isn't",True,"wo\"n't"]
        w = json.write(s)
        assert "true" in w

    def testWriteStringWithDoubleQuote(self):
        s = "do\"nt"
        w = json.write(s)
        assert w == '"do\\\"nt"'

    def testReadDictWithSlashStarComments(self):
        s = """
        {'a':false,  /*don't want b
            b:true, */
        'c':true
        }
        """
        assert json.read(s) == {'a':False,'c':True}

    def testReadDictWithTwoSlashStarComments(self):
        s = """
        {'a':false,  /*don't want b
            b:true, */
        'c':true,
        'd':false,  /*don;t want e
            e:true, */
        'f':true
        }
        """
        assert json.read(s) == {'a':False,'c':True, 'd':False,'f':True}

    def testReadDictWithDoubleSlashComments(self):
        s = """
        {'a':false,
          //  b:true, don't want b
        'c':true
        }
        """
        assert json.read(s) == {'a':False,'c':True}

    def testReadStringWithEscapedSingleQuote(self):
        s = '"don\'t tread on me."'
        assert json.read(s) == "don't tread on me."

    def testWriteStringWithEscapedDoubleQuote(self):
        s = 'he said, \"hi.'
        t = json.write(s)
        assert json.write(s) == '"he said, \\\"hi."'

    def testReadStringWithEscapedDoubleQuote(self):
        s = r'"She said, \"Hi.\""'
        assert json.read(s) == 'She said, "Hi."'

    def testReadStringWithNewLine(self):
        s = r'"She said, \"Hi,\"\n to which he did not reply."'
        assert json.read(s) == 'She said, "Hi,"\n to which he did not reply.'

    def testReadNewLine(self):
        s = r'"\n"'
        assert json.read(s) == '\n'

    def testWriteNewLine(self):
        s = u'\n'
        assert json.write(s) == r'"\n"'

    def testWriteSimpleUnicode(self):
        s = u'hello'
        assert json.write(s) == '"hello"'

    def testReadBackSlashuUnicode(self):
        s = u'"\u0066"'
        assert json.read(s) == 'f'

    def testReadBackSlashuUnicodeInDictKey(self):
        s = u'{"\u0066ather":34}'
        assert json.read(s) == {'father':34}

    def testReadDictKeyWithBackSlash(self):
        s = u'{"mo\\use":22}'
        self.assertEqual(json.read(s) , {r'mo\use':22})

    def testWriteDictKeyWithBackSlash(self):
        s = {"mo\\use":22}
        self.assertEqual(json.write(s) , r'{"mo\\use":22}')

    def testWriteListOfBackSlashuUnicodeStrings(self):
        s = [u'\u20ac',u'\u20ac',u'\u20ac']
        self.assertEqual(spaceless(json.write(s)) ,u'["\u20ac","\u20ac","\u20ac"]')

    def testWriteUnicodeCharacter(self):
        s = json.write(u'\u1001', 'ascii')
        self.assertEqual(u'"\u1001"', s)

    def testWriteUnicodeCharacter1(self):
        s = json.write(u'\u1001', 'ascii',outputEncoding='ascii')
        self.assertEqual(r'"\u1001"', s)

    def testWriteHexUnicode(self):
        s = unicode('\xff\xfe\xbf\x00Q\x00u\x00\xe9\x00 \x00p\x00a\x00s\x00a\x00?\x00','utf-16')
        p = json.write(s, 'latin-1', outputEncoding="latin-1")
        self.assertEqual(unicode(p,'latin-1'), u'"¿Qué pasa?"')

    def testWriteHexUnicode1(self):
        s = unicode('\xff\xfe\xbf\x00Q\x00u\x00\xe9\x00 \x00p\x00a\x00s\x00a\x00?\x00','utf-16')
        p = json.write(s, 'latin-1')
        self.assertEqual(p, u'"¿Qué pasa?"')

    def testWriteDosPath(self):
        s = 'c:\\windows\\system'
        assert json.write(s) == r'"c:\\windows\\system"'

    def testWriteDosPathInList(self):
        s = ['c:\windows\system','c:\\windows\\system',r'c:\windows\system']
        self.assertEqual(json.write(s) , r'["c:\\windows\\system","c:\\windows\\system","c:\\windows\\system"]')


    def readImportExploit(self):
        s = ur"\u000aimport('os').listdir('.')"
        json.read(s)

    def testImportExploit(self):
        self.assertRaises(ReadException, self.readImportExploit)

    def readClassExploit(self):
        s = ur'''"__main__".__class__.__subclasses__()'''
        json.read(s)

    def testReadClassExploit(self):
        self.assertRaises(ReadException, self.readClassExploit)

    def readBadJson(self):
        s = "'DOS'*30"
        json.read(s)

    def testReadBadJson(self):
        self.assertRaises(ReadException, self.readBadJson)

    def readUBadJson(self):
        s = ur"\u0027DOS\u0027*30"
        json.read(s)

    def testReadUBadJson(self):
        self.assertRaises(ReadException, self.readUBadJson)

    def testReadEncodedUnicode(self):
        obj = "'La Peña'"
        r = json.read(obj, 'utf-8')
        self.assertEqual(r, unicode('La Peña','utf-8'))

    def testUnicodeFromNonUnicode(self):
        obj = "'\u20ac'"
        r = json.read(obj)
        self.assertEqual(r, u'\u20ac')

    def testUnicodeInObjectFromNonUnicode(self):
        obj = "['\u20ac', '\u20acCESS', 'my\u20ACCESS']"
        r = json.read(obj)
        self.assertEqual(r, [u'\u20AC', u'\u20acCESS', u'my\u20acCESS'])

    def testWriteWithEncoding(self):
        obj = u'La Peña'
        r = json.write(obj,'latin-1',outputEncoding='latin-1')
        self.assertEqual(unicode(r,'latin-1'), u'"La Peña"')

    def testWriteWithEncodingBaseCases(self):
        #input_uni =  u"'Ă�rvĂ­ztĹąrĹ� tĂźkĂśrfĂşrĂłgĂŠp'"
        input_uni = u'\xc1rv\xedzt\u0171r\u0151 t\xfck\xf6rf\xfar\xf3g\xe9p'
        #print "input_uni is %s" % input_uni.encode('latin2')
        # the result supposes doUxxxx = False
        good_result = u'"\xc1rv\xedzt\u0171r\u0151 t\xfck\xf6rf\xfar\xf3g\xe9p"'

        # from utf8
        obj = input_uni.encode('utf-8')
        r = json.write(obj, 'utf-8',outputEncoding='utf-8')
        self.assertEqual(unicode(r,'utf-8'), good_result)

        # from unicode
        obj = input_uni
        r = json.write(obj, outputEncoding='utf-8')
        self.assertEqual(unicode(r,'utf-8'), good_result)

        # from latin2
        obj = input_uni.encode('latin2')
        r = json.write(obj, 'latin2', outputEncoding='latin2')
        self.assertEqual(unicode(r,'latin2'), good_result)

        # from unicode, encoding is ignored
        obj = input_uni
        r = json.write(obj, 'latin2', outputEncoding='latin2')
        self.assertEqual(unicode(r,'latin2'), good_result)

        # same with composite types, uni
        good_composite_result = \
        u'["\xc1rv\xedzt\u0171r\u0151 t\xfck\xf6rf\xfar\xf3g\xe9p","\xc1rv\xedzt\u0171r\u0151 t\xfck\xf6rf\xfar\xf3g\xe9p"]'
        #print "Good composite result = %s" % good_composite_result.encode('latin2')
        obj = [input_uni, input_uni]
        r = json.write(obj, outputEncoding='utf-8')
        #print "r is %s, length is %s." % (r, len(r))
        self.assertEqual(unicode(r,'utf-8'), good_composite_result)

        # same with composite types, utf-8
        obj = [input_uni.encode('utf-8'), input_uni.encode('utf-8')]
        r = json.write(obj, 'utf-8')
        # print unicode(r,'utf-8'), good_composite_result
        #self.assertEqual(unicode(r,'utf-8'), good_composite_result)

        # same with composite types, latin2
        obj = [input_uni.encode('latin2'), input_uni.encode('latin2')]
        r = json.write(obj, 'latin2')
        #cannot assertEqual here, but the printed representation should be readable
        #self.assertEqual(unicode(r,'latin2'), good_composite_result)

        # same with composite types, mixed utf-8 with unicode
        obj = [input_uni, input_uni.encode('utf-8')]
        r = json.write(obj, 'utf-8')
        #cannot assertEqual here, but the printed representation should be readable
        #self.assertEqual(unicode(r,'utf-8'), good_composite_result)

    #these tests from Koen van der Sande; just in case we really want literal
    # '\n' sent across the wire.

    def testReadSpecialEscapedChars1(self):
        test = r'"\\f"'
        self.assertEqual([ord(x) for x in json.read(test)],[92,102])

    def testReadSpecialEscapedChars2(self):
        test = r'"\\a"'
        self.assertEqual([ord(x) for x in json.read(test)],[92,97])

    def testReadSpecialEscapedChars3(self):
        test = r'"\\\\a"'
        self.assertEqual([ord(x) for x in json.read(test)],[92,92,97])

    def testReadSpecialEscapedChars4(self):
        test = r'"\\\\b"'
        self.assertEqual([ord(x) for x in json.read(test)],[92,92,98])

    def testReadSpecialEscapedChars5(self):
        test = r'"\\\n"'
        self.assertEqual([ord(x) for x in json.read(test)],[92,10])


#We'll bring up a LIVE HTTP server on localhost to be able to test
#JSONRPCProxy
#stuff taken from zc.testbrowser.tests, thanks!

class TestHandler(BaseHTTPServer.BaseHTTPRequestHandler):

    def version_string(self):
        return 'BaseHTTP'

    def date_time_string(self):
        return 'Mon, 17 Sep 2007 10:05:42 GMT'

    def do_GET(self):
        #print "GET"

        if self.path.endswith('robots.txt'):
            self.send_response(404)
            self.send_header('Connection', 'close')
            return

        global next_response_body
        global next_response_status
        global next_response_reason
        global next_response_type

        if next_response_body is None:
            self.send_response(500)
            self.send_header('Connection', 'close')
            return

        self.send_response(next_response_status)
        self.send_header('Connection', 'close')
        self.send_header('Content-type', next_response_type)
        self.send_header('Content-Length', str(len(next_response_body)))
        self.end_headers()
        self.wfile.write(next_response_body)

    def do_POST(self):
        #print "POST"

        body = self.rfile.read(int(self.headers['content-length']))

        #print body

        global last_request_body
        last_request_body = body
        global last_request_headers
        last_request_headers = self.headers

        global next_response_body
        global next_response_status
        global next_response_reason
        global next_response_type

        self.send_response(next_response_status)
        self.send_header('Connection', 'close')
        self.send_header('Content-type', next_response_type)
        self.send_header('Content-Length', str(len(next_response_body)))
        self.end_headers()
        self.wfile.write(next_response_body)

    def log_request(self, *args, **kws):
        pass

def set_next_response(
    response_body=None,
    response_status=200, response_reason='OK',
    response_type="text/html"):

    global next_response_body
    global next_response_status
    global next_response_reason
    global next_response_type

    next_response_body = response_body
    next_response_status = response_status
    next_response_reason = response_reason
    next_response_type = response_type

def set_next_response_json(result, jsonId=None, error=None):
    jsonId = jsonId or "jsonrpc"
    wrapper = {'id': jsonId}
    wrapper['result'] = result
    wrapper['error'] = error

    json = JSONWriter()
    data = json.write(wrapper)

    set_next_response(data,
        response_type="application/x-javascript;charset=utf-8"
        )



def get_last_request():
    global last_request_body
    return last_request_body

def get_last_headers():
    global last_request_headers
    return last_request_headers

def serve_requests(server):
    global json_test_server_stop
    while not json_test_server_stop:
        server.handle_request()
    server.server_close()

def setUpServer(test):
    global json_test_server_stop
    json_test_server_stop = False
    port = random.randint(20000,30000)
    test.TEST_PORT = port
    server = BaseHTTPServer.HTTPServer(('localhost', port), TestHandler)
    thread = threading.Thread(target=serve_requests, args=[server])
    thread.setDaemon(True)
    thread.start()
    test.web_server_thread = thread

def tearDownServer(test):
    global json_test_server_stop
    json_test_server_stop = True
    # make a request, so the last call to `handle_one_request` will return
    try:
        urllib.urlretrieve('http://localhost:%d/' % test.TEST_PORT)
    except IOError:
        pass # it's ok, server is already dead
    test.web_server_thread.join()


class JSONRPCProxyLiveTester(unittest.TestCase):
    def setUp(self):
        zope.component.provideUtility(JSONWriter(), IJSONWriter)
        zope.component.provideUtility(JSONReader(), IJSONReader)

        setUpServer(self)

    def tearDown(self):
        tearDownServer(self)

    def testSimple(self):
        proxy = JSONRPCProxy('http://localhost:%d/' % self.TEST_PORT)
        set_next_response_json(True, "jsonrpc")
        y = proxy.hello()
        self.assertEqual(y, True)

        x = get_last_request()
        self.assertEqual(x,
            """{"params":[],"jsonrpc":"2.0","method":"hello","id":"jsonrpc"}""")

        set_next_response_json(123, "jsonrpc")
        y = proxy.greeting(u'Jessy')
        self.assertEqual(y, 123)

        x = get_last_request()
        self.assertEqual(x,
            """{"params":["Jessy"],"jsonrpc":"2.0","method":"greeting","id":"jsonrpc"}""")

        set_next_response('blabla')
        try:
            y = proxy.hello()
        except ResponseError:
            pass
        else:
            self.fail("ResponseError expected")

        set_next_response('{blabla}')
        try:
            y = proxy.hello()
        except ResponseError:
            pass
        else:
            self.fail("ResponseError expected")

    def testSimpleBasicAuth(self):
        transport = BasicAuthTransport('user','passwd')

        proxy = JSONRPCProxy('http://localhost:%d/' % self.TEST_PORT,
                             transport=transport)
        set_next_response_json(True, "jsonrpc")
        y = proxy.hello()
        self.assertEqual(y, True)

        x = get_last_request()
        self.assertEqual(x,
            """{"params":[],"jsonrpc":"2.0","method":"hello","id":"jsonrpc"}""")
        x = get_last_headers()
        self.assertEqual(x['authorization'],
                         'Basic dXNlcjpwYXNzd2Q=')

        set_next_response_json(123, "jsonrpc")
        y = proxy.greeting(u'Jessy')
        self.assertEqual(y, 123)

        x = get_last_request()
        self.assertEqual(x,
            """{"params":["Jessy"],"jsonrpc":"2.0","method":"greeting","id":"jsonrpc"}""")

        set_next_response('blabla')
        try:
            y = proxy.hello()
        except ResponseError:
            pass
        else:
            self.fail("ResponseError expected")

        set_next_response('{blabla}')
        try:
            y = proxy.hello()
        except ResponseError:
            pass
        else:
            self.fail("ResponseError expected")

    dataToTest = [
        {'response_json': True,
         'call_method': 'hello',
         'assert_retval': True,
         'assert_request':
            """{"params":[],"jsonrpc":"2.0","method":"hello","id":"jsonrpc"}""",
        },
        {'response_json': 123,
         'call_method': 'greeting',
         'call_args': [u'Jessy'],
         'assert_retval': 123,
         'assert_request':
            """{"params":["Jessy"],"jsonrpc":"2.0","method":"greeting","id":"jsonrpc"}""",
        },
        {'response': 'blabla',
         'call_method': 'hello',
         'exception': ResponseError,
        },
        {'response': 'blabla',
         'response_status': 404,
         'call_method': 'hello',
         'exception': ProtocolError,
        },
    ]

    def testDataDriven(self):
        for item in self.dataToTest:
            jsonid = item.get('proxy_jsonid', None)
            transport = item.get('proxy_transport', None)
            proxy = JSONRPCProxy('http://localhost:%d/' % self.TEST_PORT,
                                 transport=transport,
                                 jsonId=jsonid)

            if 'response_json' in item:
                #set response based on JSON data
                error = item.get('response_json_error', None)
                jsonid = item.get('response_jsonid', None)
                set_next_response_json(item['response_json'],
                                       jsonid, error=error)
            else:
                #set response based on plain HTTP data
                response_status=item.get('response_status', 200)
                response_reason=item.get('response_reason', 'OK')
                response_type=item.get('response_type',"text/html")

                set_next_response(item['response'], response_status,
                    response_reason, response_type)

            args = item.get('call_args', [])
            kwargs = item.get('call_kwargs', {})
            exception = item.get('exception', None)
            method = getattr(proxy, item['call_method'])

            if exception:
                try:
                    retval = method(*args, **kwargs)
                except exception:
                    pass
                else:
                    self.fail("%s expected" % str(exception.__class__))

                if 'assert_request' in item:
                    x = get_last_request()
                    self.assertEqual(x, item['assert_request'])
            else:
                retval = method(*args, **kwargs)
                if 'assert_retval' in item:
                    self.assertEqual(retval, item['assert_retval'])
                if 'assert_request' in item:
                    x = get_last_request()
                    self.assertEqual(x, item['assert_request'])

def test_suite():
    return unittest.TestSuite((
        DocFileSuite('README.txt',
            optionflags=doctest.NORMALIZE_WHITESPACE|doctest.ELLIPSIS,
            ),
        unittest.makeSuite(JSONTests),
        unittest.makeSuite(JSONRPCProxyLiveTester),
        ))


if __name__=='__main__':
    unittest.main(defaultTest='test_suite')
    #unittest.TextTestRunner().run(test_suite())



##############################################################################
#
# Copyright (c) 2007 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
$Id:$
"""
__docformat__ = "reStructuredText"

import string
import urllib
import httplib
import copy
import base64
import types
import logging
import socket

import zope.component
from z3c.json import interfaces
from z3c.json.exceptions import ProtocolError
from z3c.json.exceptions import ResponseError

from z3c.json.transport import Transport
from z3c.json.transport import SafeTransport

logger = logging.getLogger(__name__)
JSON_RPC_VERSION = '2.0'


class _Method(object):

    def __init__(self, call, name, jsonId, jsonVersion):
        self.call = call
        self.name = name
        self.jsonId = jsonId
        self.jsonVersion = jsonVersion

    def __call__(self, *args, **kwargs):
        request = {}
        # add our version
        if self.jsonVersion == '1.0':
            pass
        elif self.jsonVersion == '1.1':
            request['version'] = self.jsonVersion
        else:
            request['jsonrpc'] = self.jsonVersion
        request['method'] = self.name

        # There is not support for postional and named parameters in one 
        # call. We propably will add support for this within a extension
        # in a later version. Till then, we will raise an exception if
        # we will get both paramters.
        if len(args) > 0 and len(kwargs) > 0:
            raise ValueError(
                'Mixing positional and named parameters in one call is not possible')

        if self.jsonVersion in ['1.0', '1.1']:
            if len(args) > 0:
                params = args
            elif len(kwargs) > 0:
                params = copy.copy(kwargs)
                index = 0
                for arg in args:
                    params[str(index)] = arg
                    index += 1
            else:
                params = []
        else:
            if len(args) > 0:
                params = args
            elif len(kwargs) > 0:
                params = kwargs
            else:
                params = []

        # set params and write json
        request['params'] = params
        # add our json id
        request['id'] = self.jsonId
        json = zope.component.getUtility(interfaces.IJSONWriter)
        data = json.write(request)
        try:
            return self.call(data)
        except socket.error, msg:
            raise ResponseError("JSONRPC server connection error.")

    def __getattr__(self, name):
        return _Method(self.call, "%s.%s" % (self.name, name), self.jsonId,
            self.jsonVersion)


class JSONRPCProxy(object):
    """JSON-RPC server proxy."""

    def __init__(self, uri, transport=None, encoding=None,
                 verbose=None, jsonId=None, jsonVersion=JSON_RPC_VERSION):
        utype, uri = urllib.splittype(uri)
        if utype not in ("http", "https"):
            raise IOError, "Unsupported JSONRPC protocol"
        self.__host, self.__handler = urllib.splithost(uri)
        if not self.__handler:
            self.__handler = ""

        if transport is None:
            if utype == "https":
                transport = SafeTransport()
            else:
                transport = Transport()
        self.__transport = transport

        self.__encoding = encoding
        self.__verbose = verbose
        self.jsonId = jsonId or u'jsonrpc'
        self.jsonVersion = jsonVersion or JSON_RPC_VERSION
        self.error = None

    def __request(self, request):
        """call a method on the remote server.

        This will raise a ResponseError or return the JSON result dict
        """
        # apply encoding if any
        if self.__encoding:
            request = request.encode(self.__encoding)
        # start the call
        try:
            response = self.__transport.request(self.__host, self.__handler,
                request, verbose=self.__verbose)
            self.error = None
        except ResponseError, e:
            # catch error message
            self.error = unicode(str(e), 'utf-8')
            raise

        if isinstance(response, int):
            # that's just a status code response with no result
            logger.error('Received status code %s' % response)
        elif len(response) == 3:
            # that's a valid response format
            if self.jsonId is not None and \
                (self.jsonId != response.get('id')):
                # different request id returned
                raise ResponseError("Invalid request id returned")
            if response.get('error'):
                # error mesage in response
                self.error = response['error']
                raise ResponseError("Check proxy.error for error message")
            else:
                # only return the result if everything is fine
                return response['result']

        return response

    def __getattr__(self, name):
        """This let us call methods on remote server."""
        return _Method(self.__request, name, self.jsonId, self.jsonVersion)

    def __repr__(self):
        return ("<JSONProxy for %s%s>" % (self.__host, self.__handler))

    __str__ = __repr__



##############################################################################
#
# Copyright (c) 2007 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
$Id:$
"""
__docformat__ = "reStructuredText"

import zope.interface

from z3c.json import interfaces
from z3c.json import minjson
from z3c.json import exceptions


try:
    import cjson
    hasCJson = True
except ImportError:
    import logging
    logger = logging.getLogger()
    logger.log(logging.INFO,
        "Using minjson only. cjson is much faster and available at the cheese "
        "shop. easy_install python-cjson")
    hasCJson = False


class JSONReader(object):
    """JSON reader utility."""
    zope.interface.implements(interfaces.IJSONReader)

    def read(self, aString, encoding=None):
        if hasCJson:
            try:
                # the True parameter here tells cjson to make all strings 
                # unicode. This is a good idea here.
                return cjson.decode(aString, True)
            except cjson.DecodeError:
                # fall back to minjson
                pass
        # This is a fall-back position for less-well-constructed JSON
        try:
            return minjson.read(aString, encoding)
        except minjson.ReadException, e:
            raise exceptions.ResponseError(e)


class JSONWriter(object):
    """JSON writer utility."""
    zope.interface.implements(interfaces.IJSONWriter)

    def write(self, anObject):
        if hasCJson:
            try:
                return unicode(cjson.encode(anObject))
            except cjson.EncodeError:
                # fall back to minjson
                pass
        try:
            return minjson.write(anObject)
        except minjson.WriteException, e:
            raise TypeError, e


def premarshal(data):
    """Premarshal data before handing it to JSON writer for marshalling

    The initial purpose of this function is to remove security proxies
    without resorting to removeSecurityProxy. This way, we can avoid
    inadvertently providing access to data that should be protected.
    """
    premarshaller = interfaces.IJSONRPCPremarshaller(data, alternate=None)
    if premarshaller is not None:
        return premarshaller()
    return data


class PreMarshallerBase(object):
    """Abstract base class for pre-marshallers."""
    zope.interface.implements(interfaces.IJSONRPCPremarshaller)

    def __init__(self, data):
        self.data = data

    def __call__(self):
        raise Exception, "Not implemented"


class DictPreMarshaller(PreMarshallerBase):
    """Pre-marshaller for dicts"""

    def __call__(self):
        return dict([(premarshal(k), premarshal(v))
                     for (k, v) in self.data.items()])


class ListPreMarshaller(PreMarshallerBase):
    """Pre-marshaller for list"""

    def __call__(self):
        return map(premarshal, self.data)




##############################################################################
#
# Copyright (c) 2007 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
$Id:$
"""
__docformat__ = "reStructuredText"

import string
import urllib
import httplib
import base64
import types

import zope.component
from z3c.json import interfaces
from z3c.json.exceptions import ProtocolError
from z3c.json.exceptions import ResponseError


def getparser():
    un = Unmarshaller()
    par = Parser(un)
    return par,un


class Unmarshaller(object):
    def __init__(self):
        self.data = None

    def feed(self, data):
        if self.data is None:
            self.data = data
        else:
            self.data = self.data + data

    def close(self):
        # convert to json, reader raises ResponseError on read error
        json = zope.component.getUtility(interfaces.IJSONReader)
        return json.read(self.data)


class Parser(object):
    def __init__(self, unmarshaller):
        self._target = unmarshaller
        self.data = None

    def feed(self, data):
        if self.data is None:
            self.data = data
        else:
            self.data = self.data + data

    def close(self):
        self._target.feed(self.data)


class Transport(object):
    """Handles an HTTP transaction to an JSON-RPC server.

    Standard transport class for JSON-RPC over HTTP.
    You can create custom transports by subclassing this method, and
    overriding selected methods.

    Send a complete request, and parse the response.

    @param host Target host.
    @param handler Target PRC handler.
    @param request_body JSON-RPC request body.
    @param verbose Debugging flag.
    @return Parsed response.

    """

    user_agent = "z3c.jsonrpc/0.5.0"

    def request(self, host, handler, request_body, verbose=0):
        # issue JSON-RPC request

        h = self.make_connection(host)
        if verbose:
            h.set_debuglevel(1)

        self.send_request(h, handler, request_body)
        self.send_host(h, host)
        self.send_user_agent(h)
        self.send_content(h, request_body)

        errcode, errmsg, headers = h.getreply()

        if errcode != 200:
            raise ProtocolError(host + handler, errcode, errmsg, headers)

        self.verbose = verbose

        self.parse_response_headers(headers)

        try:
            sock = h._conn.sock
        except AttributeError:
            sock = None

        return self._parse_response(h.getfile(), sock)

    def getparser(self):
        """Get parser and unmarshaller."""
        return getparser()

    def get_host_info(self, host):
        """Get authorization info from host parameter

        Host may be a string, or a (host, x509-dict) tuple; if a string,
        it is checked for a "user:pw@host" format, and a "Basic
        Authentication" header is added if appropriate.

        @param host Host descriptor (URL or (URL, x509 info) tuple).
        @return A 3-tuple containing (actual host, extra headers,
        x509 info).  The header and x509 fields may be None.

        """
        x509 = {}
        if isinstance(host, types.TupleType):
            host, x509 = host

        auth, host = urllib.splituser(host)

        if auth:
            auth = base64.encodestring(urllib.unquote(auth))
            auth = string.join(string.split(auth), "") # get rid of whitespace
            extra_headers = [
                ("Authorization", "Basic " + auth)
                ]
        else:
            extra_headers = None

        return host, extra_headers, x509

    def make_connection(self, host):
        # create a HTTP connection object from a host descriptor
        host, extra_headers, x509 = self.get_host_info(host)
        return httplib.HTTP(host)

    def send_request(self, connection, handler, request_body):
        connection.putrequest("POST", handler)

    def send_host(self, connection, host):
        host, extra_headers, x509 = self.get_host_info(host)
        connection.putheader("Host", host)
        if extra_headers:
            if isinstance(extra_headers, dict):
                extra_headers = extra_headers.items()
            for key, value in extra_headers:
                connection.putheader(key, value)

    def send_user_agent(self, connection):
        connection.putheader("User-Agent", self.user_agent)

    def send_content(self, connection, request_body):
        connection.putheader("Content-Type", "application/json")
        connection.putheader("Content-Length", str(len(request_body)))
        connection.endheaders()
        if request_body:
            connection.send(request_body)

    def parse_response_headers(self, headers):
        pass

    def parse_response(self, file):
        # compatibility interface
        return self._parse_response(file, None)

    def _parse_response(self, file, sock):
        # read response from input file/socket, and parse it

        p, u = self.getparser()

        while 1:
            if sock:
                response = sock.recv(1024)
            else:
                response = file.read(1024)
            if not response:
                break
            if self.verbose:
                print "body:", repr(response)
            p.feed(response)

        file.close()
        p.close()

        return u.close()


class SafeTransportMixin(object):
    """Handles an HTTPS transaction to an JSON-RPC server."""

    def make_connection(self, host):
        """Create a HTTPS connection object from a host descriptor

        host may be a string, or a (host, x509-dict) tuple

        """
        host, extra_headers, x509 = self.get_host_info(host)
        try:
            HTTPS = httplib.HTTPS
        except AttributeError:
            raise NotImplementedError(
                "your version of httplib doesn't support HTTPS"
                )
        else:
            return HTTPS(host, None, **(x509 or {}))

class SafeTransport(SafeTransportMixin, Transport):
    """Handles an HTTPS transaction to an JSON-RPC server."""

class BasicAuthTransport(Transport):
    """Handles a transaction to an JSON-RPC server using HTTP basic auth."""

    def __init__(self, username=None, password=None, verbose=0):
        self.username=username
        self.password=password
        self.verbose=verbose

    def send_content(self, connection, request_body):
        # send basic auth
        if self.username is not None and self.password is not None:
            connection.putheader("AUTHORIZATION", "Basic %s" %
                base64.encodestring("%s:%s" % (self.username, self.password)
                    ).replace("\012", ""))

        super(BasicAuthTransport, self).send_content(connection, request_body)

class SafeBasicAuthTransport(SafeTransportMixin, BasicAuthTransport):
    """Basic AUTH through HTTPS"""


##############################################################################
#
# Copyright (c) 2007 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
$Id:$
"""
__docformat__ = "reStructuredText"

import zope.component

from z3c.json import interfaces
from z3c.json import converter


def setUpJSONConverter():
    zope.component.provideUtility(converter.JSONReader(), interfaces.IJSONReader)
    zope.component.provideUtility(converter.JSONWriter(), interfaces.IJSONWriter)
    zope.component.provideAdapter(converter.ListPreMarshaller, (list,))
    zope.component.provideAdapter(converter.ListPreMarshaller, (tuple,))
    zope.component.provideAdapter(converter.DictPreMarshaller, (dict,))



##############################################################################
#
# Copyright (c) 2007 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
$Id:$
"""
__docformat__ = "reStructuredText"

import zope.interface


class IJSONReader(zope.interface.Interface):
    """JSON reader."""

    def read(aString):
        """read and interpret a string in JSON as python"""
        

class IJSONWriter(zope.interface.Interface):
    """JSON writer."""

    def write(anObject, encoding=None):
        """return a JSON unicode string representation of a python object
           Encode if encoding is provided.
        """
    

class IJSONRPCPremarshaller(zope.interface.Interface):
    """Premarshaller to remove security proxies"""

    def __call__():
        """return the object without proxies"""
